#if !defined(OEMCONFIG_H)
#define OEMCONFIG_H  1
/*======================================================
FILE:  OEMConfig.h

SERVICES:  OEM Configuration Interface Function

GENERAL DESCRIPTION:

This file contains prototypes and definitions for
access to OEM configuration interfaces and structures.

PUBLIC CLASSES:

None.

INITIALIZATION AND SEQUENCING REQUIREMENTS
None.

        Copyright ?1999-2002 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/
#include "AEEConfig.h"

#if defined(__cplusplus)
extern "C" {
#endif

//*******************************************************************************
//
// Static Functions - These functions can be called from statically linked code.
//
//*******************************************************************************

AEE_EXPORTS extern int  OEM_GetConfig(AEEConfigItem i, void * pBuff, int nSize);
extern int              OEM_SetConfig(AEEConfigItem i, void * pBuff, int nSize);
extern void             OEM_GetDeviceInfo(AEEDeviceInfo * pi);
extern int              OEM_GetDeviceInfoEx(AEEDeviceItem nItem, void *pBuff, int *nSize);
extern const char *     OEM_GetPath(uint16 wType);
extern boolean          OEM_GetItemStyle(AEEItemType t, AEEItemStyle * pNormal, AEEItemStyle * pSel);

#if defined(__cplusplus)
}
#endif


/*=======================================================================
  OEM  DOCUMENTATION
=======================================================================

Function: OEM_GetConfig()


Description:
   This function retrieves handset configuration information.

Prototype:
   int OEM_GetConfig(AEEConfigItem i, void * pBuff, int nSize);

Parameters:
   AEEConfigItem i: item that needs to be retrieved. It can be one of the
   following values:

===pre>
      - CFGI_DNS_IP1: 32-bit main Domain Name Server (DNS) IP Addresses in
                        network byte-order
      - CFGI_DNS_IP2: 32-bit backup Domain Name Server (DNS) IP Address in
                        network byte-order
      - CFGI_DOWNLOAD: information pertinent to the download service with the
        following information:
        + dwCarrierID: 32-bit carrier ID
        + dwPlatformID: 32-bit handset platform ID
        + bBKey: string of OEM programmed B-Key or all zeros
        + bAKey: string of SSD_A derived from A-Key or all zeros
        + szServer: string of server name
        + wFlags: 16-bit download related flags which can have one or more of the following
          values set:
          * DIF_USE_A_KEY: if set, use A-Key; otherwise use B-Key
          * DIF_TEST_ALLOWED: if set, the handset can be used to test local apps
          * DIF_MIN_FOR_SID: if set, use the MIN for the SID
          * DIF_PREPAY: if set, it is a prepay phone
          * DIF_NO_AUTO_ACK: if set, do not force ACKs until user runs MobileShop
          * DIF_SID_ENCODE: Uses SID rather than ESN for app encoding
          * DIF_SID_VALIDATE_ALL: Validates all apps rather than just SSN apps
          * DIF_RUIM_DEL_OVERRIDE: Allows one RUIM user to delete apps owned by another
          * DIF_AUTO_UPGRADE: Perform automatic upgrade when new uses purchased
          * DIF_NO_LAUNCH_MOD_ACK_DISABLED: If set, do not start an app with flag MOD_ACK_DISABLED set
        + nAuth: download authentication policy which can be one of the following values:
          * APOLICY_NONE: No authentication required
          * APOLICY_SID: User's "subscriber ID" is passed to ADS before any set of transactions started
          * APOLICY_TEXT: User should be prompted for text "key" and this sent to ADS
          * APOLICY_NUM- User should be prompted for numeric "key" and this sent to ADS
        + nPolicy: privacy policy which determines the type of certification required
          to run applet on the handset:
          * PPOLICY_BREW: TRUE-BREW-signed applet only
          * PPOLICY_CARRIER: carrier-signed applet only
          * PPOLICY_BREW_AND_CARRIER: TRUE-BREW- and carrier-signed applet only
          * PPOLICY_BREW_OR_CARRIER: TRUE-BREW- or carrier-signed applet
      - CFGI_SUBSCRIBERID: 32-byte subscriber ID in ASCII
      - CFGI_MOBILEINFO: information about the handset which include the following:
        + nCurrNAM: 8-bit handset's NAM
        + dwESN: 32-bit handset's ESN
        + szMobileID: 16-byte mobile number which consists of the following components:
          * mcc: 3-digit mobile country code
          * mnc: 2-digit mobile network code
          * min2: 3-digit mobile area code
          * min1: 7-digit mobile phone number
      - CFGI_AUTOSTART: class ID of the applet to be auto-started when AEE is initialized
        through AEE_Init().
      - CFGI_BUSY_CURSOR_OFFSET: offset position of the hourglass from the center of the
        screen. It is of the type AEERect which has the following fields:
        + x: x-offset from the center of the screen
        + y: y-offset from the center of the screen
        + dx: ignored
        + dy: ignored
      - CFGI_DOWNLOAD_BUFFER:32-bit unsigned integer value for the download buffer size, in bytes.
        The default is 10 kilobytes.
      - CFGI_DOWNLOAD_CACHE:32-bit unsigned integer value for the download buffer cache, in bytes.  Used to configure the maximum download page size that can be cached.
        The default is 1kilobytes and maximum is 10 kilobytes.
      - CFGI_HTTP_BUFFER: 32-bit unsigned integer value for the HTTP read buffer size, in bytes.
        The default is 4 kilobytes.
      - CFGI_NET_CONNTIMEOUT: 32-bit unsigned integer value for the network connection timeout,
        in milliseconds. The default is 30 seconds.
      - CFGI_SUBSCRIBERID_LEN: 32-bit signed integer value for size in bytes of subscriber ID
        The default length is 32.
      - CFGI_MAX_DISPATCH_TIME: 32-bit unsigned integer value for the maximum time BREW should
        spend in the dispatcher before relinquishing control. The default is 250 msecs.
      - CFGI_MIN_IDLE_TIME: 32-bit unsigned integer value for the minimum time BREW must relinquish
        from dispatcher. The default is 35 msecs.
      - CFGI_SLEEP_TIMER_RESOLUTION: 32-bit unsigned integer value for timer resolution during when
        processor/os is in SLEEP mode. The default is 1.2 seconds.
      - CFGI_CLIPBOARD_SIZE: 32-bit unsigned integer value for size in bytes of pre-allocated
        low-memory area for clipboard. The default is 2K bytes.
      - CFGI_SYSMEM_SIZE: 32-bit unsigned integer value for size in bytes reserved to the system
        in low-memory. The default is 2K bytes.
      - CFGI_DOWNLOAD_FS_INFO: DLItemSize structure, only the dwFSAvail and dwFSSize will be used. If
        this is not supported by the OEM, BREW will try to calculate available space with OEMFS functions.
      - CFGI_SCREEN_SAVER: AEEScreenSaverInfo structure, this is used to configure screensaver settings.
        Default values are: wSecs:30seconds, wSecsMin:5seconds, wSecsMax:300seconds.
      - CFGI_CLOSE_KEYS: OEMCloseKeys structure, the default values will be obtained from AEEDeviceInfo
        otherwise will be set to the following defaults:   wCloseApp:AVK_CLR, evtCloseApp:EVT_KEY, wCloseAllApps:AVK_END,
        evtCloseAllApps:EVT_KEY. The OEMCloseKeys structure will be filled before calling OEM_GetConfig, so you can check
        the defaults and change if necessary.
      - CFGI_FILE_CACHE_INFO: OEMFileCacheInfo structure, used to determine file caching levels.
        The default values are nCacheDefault:4k, nCacheMin:-1,nCacheMax:-1
      - CFGI_PRELOAD_COUNT: OEMs must specify the count of the PreLoaded apps
      expected on this device. The parameter is an int * that must be filled with the
      exact count on return. On a test-enabled device, this value must be set to 0.
      On a Non test-enabled device,this value must exactly match the number of PreLoaded apps
      expected to be shipped on the device. If the counts do not match, all of the PreLoaded
      apps on the device will be deleted when the handset is powered on. The way to
      preload apps on the device is to the following: 1. Set this count to the exact
      value 2. Power Cycle the Device. 3. Preload the exact number of apps on the
      device using AppLoader or EFS-Explorer like device. 4. Power Cycle the Phone.
      - CFGI_MAX_DEMO_ITEMS: Count of demo apps/items supported on the handset.  This
      number is used to store a list of previously downloaded demo items.  The default
      size of the list is 20 entries.  This means that at least 19 other demos must
      be downloaded before the same demo can be downloaded again.
      - CFGI_GPSONE_TRANSPORT: uint32 OEM GPS transport method (IP, Data burst)
      - CFGI_GPSONE_SVRIP: uint32 GPS server IP address
      - CFGI_GPSONE_SVRPORT: uint32 GPS server IP port
      - CFGI_GPSONE_LOCK: GPS lock
      - CFGI_MODULE_FSLIMIT: MIFFSLimit structure to indicate Maximum File usage and Maximum Space usage
        for the module. Default is unlimited space usage.
      - CFGI_ISTATIC_SCROLL: 32-bit unsigned integer value for the auto-scrolling speed of each line
        of text in the IStatic control, in milliseconds. The default is 1 second per visible line.
        This is useful for reducing flickering that may happen when the default value is too low/high.
      - CFGI_APP_KEY_1: OEMConfigData structure for a secret to be used by applications to authenticate
        or sign content.  This is sensitive key material, and so should not be stored in
        predictable locations (i.e. file system or fixed memory locations).
      - CFGI_CACHED_RESOURCES: Number of resource files to cache in the BREW resource file layer manages.
        The default value for this number is 5.  However, on devices with more RAM and file handles, increasing
        this value may greatly improve the usability of the device.
      - CFGI_CARDID: The Card ID of the RUIM/USIM card (if any) on the device.
      - CFGI_CARDID_LEN: Length of the card ID of the RUIM/USIM card (if any) on the device.
      - CFGI_MOBILE_DIR_NUM: The Mobile Directory Number (MDN) for this device. As per NANPA specification
        this number should be 10 digit number. It should be usable as is through BREW APIs to make a voice call.
      - CFGI_MOBILE_DIR_NUM_LEN: The length of the MDN string.
      - CFGI_FIRMWARE_ID: OEM specified Firmware ID for this device.
      - CFGI_FIRMWARE_ID_LEN: Length of the Firmware ID string.
      - CFGI_MAX_ACK_ATTEMPTS: Max retries attempted to flush the preinstall/delete acks
      - CFGI_DORMANCY_TIMEOUT_DEFAULT: 8-bit unsigned integer value (uint8) for the default 
        packet data inactivity timeout (which initiates dormancy), in seconds. 0 means inactivity timeout disabled.
        Default is 30 seconds.
      - CFGI_GPSONE_LOCATION_SERVER_NAME1: Fully qualified name of location server. If provided will
        be resolved and used by position determination process when application choses to use the default
        server.
      - CFGI_GPSONE_LOCATION_SERVER_NAME2: Fully qualified name of location server. If provided will
        be used by position determination process when CFGI_GPSONE_LOCATION_SERVER_NAME1 cannot be resolved.
===/pre>
   void * pBuff: buffer in which the return values are to be stored
   int nSize: size of the buffer

Return Value:
   0 on Success
        EFAILED or EBADPARM if the buffer is not big enough to hold the data to be returned.

Comments:
   None

Side Effects:
   None

See Also:
   IDOWNLOAD_GetConfigItem()
==================================================================
Function: OEM_SetConfig()

Description:
   This function sets new handset configuration information.

Prototype:
   int OEM_SetConfig(AEEConfigItem i, void * pBuff, int nSize);

Parameters:
   AEEConfigItem i: item whose new value needs to be set. It can be one of the
   following values:

===pre>
      - CFGI_DNS_IP1: 32-bit main Domain Name Server (DNS) IP Addresses in
                        network byte-order
      - CFGI_DNS_IP2: 32-bit backup Domain Name Server (DNS) IP Address in
                        network byte-order
      - CFGI_DOWNLOAD: information pertinent to the download service with the
        following information:
        + dwCarrierID: 32-bit carrier ID
        + dwPlatformID: 32-bit handset platform ID
        + bBKey: string of OEM programmed B-Key or all zeros
        + bAKey: string of SSD_A derived from A-Key or all zeros
        + szServer: string of server name
        + wFlags: 16-bit download related flags which can have one or more of the following
          values set:
          * DIF_USE_A_KEY: if set, use A-Key; otherwise use B-Key
          * DIF_TEST_ALLOWED: if set, the handset can be used to test local apps
          * DIF_MIN_FOR_SID: if not set, use the MIN for the SID
          * DIF_PREPAY: if set, it is a prepay phone
          * DIF_NO_AUTO_ACK: if set, do not force ACKs until user runs MobileShop
          * DIF_SID_ENCODE: Uses SID rather than ESN for app encoding
          * DIF_SID_VALIDATE_ALL: Validates all apps rather than just SSN apps
          * DIF_RUIM_DEL_OVERRIDE: Allows one RUIM user to delete apps owned by another
          * DIF_AUTO_UPGRADE: Perform automatic upgrade when new uses purchased
          * DIF_NO_LAUNCH_MOD_ACK_DISABLED: If set, do not start an app with flag MOD_ACK_DISABLED set
        + nAuth: download authentication policy which can be one of the following values:
          * APOLICY_NONE: No authentication required
          * APOLICY_SID: User's "subscriber ID" is passed to ADS before any set of transactions started
          * APOLICY_TEXT: User should be prompted for text "key" and this sent to ADS
          * APOLICY_NUM- User should be prompted for numeric "key" and this sent to ADS
        + nPolicy: privacy policy which determines the type of certification required
          to run applet on the handset:
          * PPOLICY_BREW: TRUE-BREW-signed applet only
          * PPOLICY_CARRIER: carrier-signed applet only
          * PPOLICY_BREW_AND_CARRIER: TRUE-BREW- and carrier-signed applet only
          * PPOLICY_BREW_OR_CARRIER: TRUE-BREW- or carrier-signed applet
      - CFGI_SUBSCRIBERID: 32-byte subscriber ID in ASCII
      - CFGI_MOBILEINFO: information about the handset which include the following:
        + nCurrNAM: 8-bit handset's NAM
        + dwESN: 32-bit handset's ESN
        + szMobileID: 16-byte mobile number which consists of the following components:
          * mcc: 3-digit mobile coutry code
          * mnc: 2-digit mobile network code
          * min2: 3-digit mobile area code
          * min1: 7-digit mobile phone number
      - CFGI_AUTOSTART: class ID of the applet to be auto-started when AEE is initialized
        through AEE_Init().
      - CFGI_BUSY_CURSOR_OFFSET: offset position of the hourglass from the center of the
        screen. It is of the type AEERect which has the following fields:
        + x: x-offset from the center of the screen
        + y: y-offset from the center of the screen
        + dx: ignored
        + dy: ignored
      - CFGI_DOWNLOAD_BUFFER:32-bit unsigned integer value for the download buffer size, in bytes.
        The default is 10 kilobytes.
      - CFGI_HTTP_BUFFER: 32-bit unsigned integer value for the HTTP read buffer size, in bytes.
        The default is 4 kilobytes.
      - CFGI_NET_CONNTIMEOUT: 32-bit unsigned integer value for the network connection timeout,
        in milliseconds. The default is 30 seconds.
      - CFGI_SUBSCRIBERID_LEN: 32-bit signed integer value for size in bytes of subscriber ID
        The default length is 32.
      - CFGI_MAX_DISPATCH_TIME: 32-bit unsigned integer value for the maximum time BREW should
        spend in the dispatcher before relinquishing control. The default is 250 msecs.
      - CFGI_MIN_IDLE_TIME: 32-bit unsigned integer value for the minimum time BREW must relinquish
        from dispatcher. The default is 35 msecs.
      - CFGI_SLEEP_TIMER_RESOLUTION: 32-bit unsigned integer value for timer resolution during when
        processor/os is in SLEEP mode. The default is 1.2 seconds.
      - CFGI_CLIPBOARD_SIZE: 32-bit unsigned integer value for size in bytes of pre-allocated
        low-memory area for clipboard. The default is 2K bytes.
      - CFGI_SYSMEM_SIZE: 32-bit unsigned integer value for size in bytes reserved to the system
        in low-memory. The default is 2K bytes.
      - CFGI_DOWNLOAD_FS_INFO: DLItemSize structure, only the dwFSAvail and dwFSSize will be used. If
        this is not supported by the OEM, BREW will try to calculate available space with OEMFS functions.
      - CFGI_SCREEN_SAVER: AEEScreenSaverInfo structure, this is used to configure screensaver settings.
        Default values are: wSecs:30seconds, wSecsMin:5seconds, wSecsMax:300seconds.
      - CFGI_CLOSE_KEYS: OEMCloseKeys structure, the default values will be obtained from AEEDeviceInfo
        otherwise will be set to the following defaults:   wCloseApp:AVK_CLR, evtCloseApp:EVT_KEY, wCloseAllApps:AVK_END,
        evtCloseAllApps:EVT_KEY. The OEMCloseKeys structure will be filled before calling OEM_GetConfig, so you can check
        the defaults and change if necessary.
   - CFGI_PRELOAD_COUNT: OEMs must store the count of the PreLoaded apps passed
      to this function and must return this exact same value whenever OEM_GetConfig()
      is invoked with CFGI_PRELOAD_COUNT. This value must be stored in a persistent
      manner so that the same value can be remembered and returned whenever OEM_GetConfig()
      is invoked (including after power-cycling the phone).
      Failure to store this value in a persistent manner and return the same value is
      a FATAL error and may cause all of the pre-loaded apps on the device to be deleted.

      - CFGI_FILE_CACHE_INFO: OEMFileCacheInfo structure, used to determine file caching levels.
        The default values are nCacheDefault:4k, nCacheMin:-1,nCacheMax:-1.
      - CFGI_GPSONE_TRANSPORT: uint32 OEM GPS transport method (IP, Data burst)
      - CFGI_GPSONE_SVRIP: uint32 GPS server IP address
      - CFGI_GPSONE_SVRPORT: uint32 GPS server IP port
      - CFGI_GPSONE_LOCK: GPS lock
      - CFGI_MODULE_FSLIMIT: MIFFSLimit structure to indicate Maximum File usage and Maximum Space usage
        for the module. Default is unlimited space usage.
      - CFGI_ISTATIC_SCROLL: 32-bit unsigned integer value for the auto-scrolling speed of each line
        of text in the IStatic control, in milliseconds. The default is 1 second per visible line.
        This is useful for reducing flickering that may happen when the default value is too low/high.
      - CFGI_OFFLINE_PPP_TIMEOUT: 32-bit unsigned integer value for the offline PPP timeout, in
        milliseconds. If a PPP session is in progress when the call state goes offline, PPP will be
        closed after this timeout, if the phone call state remains offline.  The default is 60 seconds.
      - CFGI_KB_AUTOREPEAT: Pointer to OEMKBAutoRepeat structure to allow OEM to control keypad auto-repeat
        and associated repeat rates.  OEM can control both the delay prior to the first repeat
        (OEMKBAutoRepeat.dwStart) as well as the repeat rate (OEMKBAutoRepeat.dwRate).  If OEMKBAutoRepeat.dwStart
        is 0, auto-repeat is disabled.  If OEMKBAutoRepeat.dwRate is 0, the EVT_KEY event will repeat only
        a single time.
      - CFGI_DORMANCY_TIMEOUT_DEFAULT: 8-bit unsigned integer value (uint8) for the default 
        packet data inactivity timeout (which initiates dormancy), in seconds. 0 means inactivity timeout disabled.
        Default is 30 seconds.
      - CFGI_MANNER_MODE: AEE_MANNER_MODE_XXX value (uint32) to set the manner mode of the device.
===/pre>
   void * pBuff: buffer in which the new values are stored
   int nSize: size of the buffer

Return Value:
   0 on Success

Comments:
   None

Side Effects:
   None

See Also:
   Function: OEM_GetConfig()

==================================================================
Function: OEM_GetDeviceInfo()

Description:
   This function retrieves current handset's physical and hardware
characteristics.

Prototype:
   void OEM_GetDeviceInfo(AEEDeviceInfo * pi);

Parameters:
   AEEDeviceInfo * pi: retrieved buffer where AEEDeviceInfo
   (refer BREW SDK API Reference Guide) is stored.

Return Value:
   None

Comments:
   None

Side Effects:
   None

See Also:
   None

==================================================================
Function: OEM_GetDeviceInfoEx()

Description:
   This function is used to get specific information about the device. It takes
   an ID that specifies what information is needed. The buffer contains the corresponding
   information on return.

Prototype:

   int   OEM_GetDeviceInfoEx(AEEDeviceItem nItem, void *pBuff, int *pnSize);

Parameters:
   nItem: Specifies the Item whose info is needed. See documentation on
   AEEDeviceItem for the supported device items.
   pBuff: Contains the corresponding information on return.
   pnSize: Contains the size of pBuff. On return, it contains the size filled. This parameter may
   be NULL for certain device Items.

Return Value:
   SUCCESS: if successful
   EBADPARM: if bad parameters are passed in
   EUNSUPPORTED: if this item is not supported

Comments:
   None

Side Effects:
   None

See Also:
   AEEDeviceItem
   ISHELL_GetDeviceInfoEx()

=======================================================================
Function: OEM_GetPath()

Description:
   This function is called by the AEE to determine the path for the
directory names used by BREW. BREW uses six directories:
   - APPS: the directory where all the MIFs and
            applet directories are stored
   - SHARED: where all the shared files are
             stored
   - RINGER: where all the ringer files are
             stored
   - ADDRBOOK: where all of the address book
               files are stored
   - MIF: where all of the MIFs are stored
   - LOGO: where all of the LOGO files are stored

The path must not have a trailing directory separation character.

Prototype:

   const char * OEM_GetPath(uint16 wType)

Parameters:
   uint16 wType: any of the six directories used by the AEE: APPS, ADDRBOOK, MIF,
                 RINGER, SHARED or LOGO

Return Value:
   Returns const char * pointer to path

Comments:
   None

See Also:
   OEM_GetAppPath()
   OEM_GetMIFPath()
   OEM_GetRingerPath()
   OEM_GetSharedPath()
   OEM_GetAddrBookPath()
   OEM_GetLogoPath()

==================================================================
Function: OEM_GetSharedPath ()

Description:
   This function is called by the AEE to determine the path for the
shared directory. This is the directory that all the applets with
the shared directory access privilege share. The path must not have
a trailing directory separation character.

Prototype:

   const char * OEM_GetSharedPath(void)

Parameters:
   None

Return Value:
   Returns const char * pointer to path.

Comments:
   None

See Also:
   OEM_GetPath()

==================================================================
Function: OEM_GetAppPath ()

Description:
   This function is called by the AEE to determine the path for the MIFs
(Module Information Files). This is the directory that the AppManager shall
look for the MIFs. The path must not have a trailing directory separation
character.

Prototype:

   const char * OEM_GetAppPath(void)

Parameters:
   None

Return Value:
   Returns const char * pointer to path.

Comments:
   None

See Also:
   OEM_GetPath()

==================================================================
Function: OEM_GetRingerPath ()

Description:
   This function is called by the AEE to determine the path for the
ringers. This is the directory where all the ringer files are stored.
The path must not have a trailing directory separation character.

Prototype:

   const char * OEM_GetRingerPath(void)

Parameters:
   None

Return Value:
   Returns const char * pointer to path.

Comments:
   None

See Also:
   OEM_GetPath()

==================================================================
Function: OEM_GetAddrBookPath ()

Description:
   This function is called by the AEE to determine the path for the
address book application and files. The path must not have a trailing
directory separation character.

Prototype:

   const char * OEM_GetAddrBookPath(void)

Parameters:
   None

Return Value:
   Returns const char * pointer to path.

Comments:
   None

See Also:
   OEM_GetPath()

==================================================================
Function: OEM_GetMIFPath ()

Description:
   This function is called by the AEE to determine the path for the
Module Information Files (MIFs). The path is the directory in which
the AppManager looks for MIFs. The path must not have a trailing
directory separation character.

Prototype:

   const char * OEM_GetMIFPath(void)

Parameters:
   None

Return Value:
   Returns const char * pointer to path.

Comments:
   None

See Also:
   OEM_GetPath()

==================================================================
Function: OEM_GetLogoPath ()

Description:
   This function is called by the AEE to determine the path for the
LOGO files. The path must not have a trailing directory separation
character.

Prototype:

   const char * OEM_GetLogoPath(void)

Parameters:
   None

Return Value:
   Returns const char * pointer to path.

Comments:
   None

See Also:
   OEM_GetPath()

==================================================================*/


#endif // AEE_CONFIG_H
