/*======================================================
FILE:  bridle_GSM1xCardHandler.c

SERVICES:  BRIDLE SWI Handler for GSM1xCardHandler.

GENERAL DESCRIPTION:
   Implements the supervisor mode handler for BRIDLE GSM1xCardHandler

PUBLIC CLASSES AND STATIC FUNCTIONS:

INITIALIZATION & SEQUENCING REQUIREMENTS:

   See Exported Routines

        Copyright  1999-2003 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/
#include "BRIDLEInit.h"
#include "OEMFeatures.h"
#if defined(FEATURE_GSM1x)

#define _BRIDLE_GSM1XCARDHANDLER_IMPL
#include "bridle_GSM1xCardHandler.h"
#include "AEEBRIDLE.h"

#if __ARMCC_VERSION >= 120000
#pragma arm section zidata = "bridle_svc_rw_usr_ro"
#endif
uint32 bridle_oemGSM1xCardHandler_swi;
#if __ARMCC_VERSION >= 120000
#pragma arm section zidata
#endif


/* List of handlers that patch calls to actual routines. The order 
** corresponds to the subroutine ids enumerated in the enum in bridle_GSM1xCardHandler.h */
static const PFNSWIHANDLER gPtrTable[BRIDLE_GSM1XCARDHANDLER_MAX] = { 
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_Init,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_IsSMSInit,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_IsSMSEnabled,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetSMSStatus,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetNumSMSSlots,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetSms,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_SetSms,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ClearSms,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ClearAllSms,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadSMSCAddress,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_SetSMSCAddress,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadSMSS,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_SetSMSS,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadForbiddenPLMN,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_WriteForbiddenPLMN,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadPLMNSelector,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_WritePLMNSelector,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadAccessClass,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadICCId,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadIMSI,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadMSISDN,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_ReadAD,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetDirPresence,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetAdn,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_SetAdn,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetNumAdnRec,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xCardHandler_SVC_GetAdnAlphaSize
};

/*===========================================================================

FUNCTION   
   bridle_GSM1xCardHandler_swihandler

DESCRIPTION
   Invokes the appropriate unpack routine and passes its arguments.
  
DEPENDENCIES
   This is invoked by SWI handler.

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
uint32 bridle_GSM1xCardHandler_swihandler( uint32 arg1, uint32 arg2, uint32 arg3, uint32 arg4, uint32 arg5 )
{
   return bridle_generic_swihandler( arg1, arg2, arg3, arg4, arg5, 
                                     BRIDLE_GSM1XCARDHANDLER_MAX, gPtrTable );
}

/*===========================================================================

FUNCTION   
   RegisterGSM1xCardHandlerForSWI

DESCRIPTION
   Registers with SWI handler and obtains ID for GSM1xCardHandler Module.
  
DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int RegisterGSM1xCardHandlerForSWI(void *po)
{
   return bridle_RegisterSubSystem( bridle_GSM1xCardHandler_swihandler, &bridle_oemGSM1xCardHandler_swi );
}

#endif // FEATURE_GSM1x
