/*===========================================================================

FILE: TestHtmlViewer.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEHtmlViewer.h"		// HtmlViewer interface definitions
#include "AEEFile.h"					// File interface definitions
#include "AEEStdLib.h"

#include "TestHtmlViewer.bid"

#define		PASS_HTML			"Password.html"		// dbԍ͉


/*-------------------------------------------------------------------
Applet structure. All variables in here are reference via "pMe->"
-------------------------------------------------------------------*/
// create an applet structure that's passed around. All variables in
// here will be able to be referenced as static.
typedef struct _TestHtmlViewer {
	AEEApplet      a ;	       // First element of this structure must be AEEApplet
    AEEDeviceInfo  DeviceInfo; // always have access to the hardware device information	
	AEERect				aRect;						// ʗ̈
	IHtmlViewer			*m_pHtmlViewer;				// IHtmlViewer
	IFileMgr			*m_pFileMgr;				// IFileMgrCX^X
    // add your own variables here...



} TestHtmlViewer;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static  boolean TestHtmlViewer_HandleEvent(TestHtmlViewer* pMe, AEEEvent eCode, 
                                             uint16 wParam, uint32 dwParam);
boolean TestHtmlViewer_InitAppData(TestHtmlViewer* pMe);
void    TestHtmlViewer_FreeAppData(TestHtmlViewer* pMe);
void LoadFiles(TestHtmlViewer *pApp, char *pFileName);
void HtmlNotify_MENU(TestHtmlViewer *pApp, HViewNotify *phvn, AEEEvent eCode, uint16 wParam, uint32 dwParam);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================
FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell *pIShell, IModule *po, void **ppObj)
{
	*ppObj = NULL;

	if( ClsId == AEECLSID_TESTHTMLVIEWER )
	{
		// Create the applet and make room for the applet structure
		if( AEEApplet_New(sizeof(TestHtmlViewer),
                          ClsId,
                          pIShell,
                          po,
                          (IApplet**)ppObj,
                          (AEEHANDLER)TestHtmlViewer_HandleEvent,
                          (PFNFREEAPPDATA)TestHtmlViewer_FreeAppData) ) // the FreeAppData function is called after sending EVT_APP_STOP to the HandleEvent function
                          
		{
			//Initialize applet data, this is called before sending EVT_APP_START
            // to the HandleEvent function
			//_asm int 3;
			if(TestHtmlViewer_InitAppData((TestHtmlViewer*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

        } // end AEEApplet_New

    }

	return(EFAILED);
}


/*===========================================================================
FUNCTION SampleAppWizard_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SampleAppWizard_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean TestHtmlViewer_HandleEvent(TestHtmlViewer* pMe, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  

    switch (eCode) 
	{
        // App is told it is starting up
        case EVT_APP_START:                        
		    // Add your code here...
			_asm int 3;
			LoadFiles(pMe, (char*)PASS_HTML);
            return(TRUE);


        // App is told it is exiting
        case EVT_APP_STOP:
            // Add your code here...

      		return(TRUE);


        // App is being suspended 
        case EVT_APP_SUSPEND:
		    // Add your code here...

      		return(TRUE);


        // App is being resumed
        case EVT_APP_RESUME:
		    // Add your code here...

      		return(TRUE);


        // An SMS message has arrived for this app. Message is in the dwParam above as (char *)
        // sender simply uses this format "//BREW:ClassId:Message", example //BREW:0x00000001:Hello World
        case EVT_APP_MESSAGE:
		    // Add your code here...

      		return(TRUE);

        // A key was pressed. Look at the wParam above to see which key was pressed. The key
        // codes are in AEEVCodes.h. Example "AVK_1" means that the "1" key was pressed.
        case EVT_KEY:
		    // Add your code here...

      		return(TRUE);


        // If nothing fits up to this point then we'll just break out
        default:
            break;
   }

   return FALSE;
}


// this function is called when your application is starting up
boolean TestHtmlViewer_InitAppData(TestHtmlViewer* pMe)
{
	int						nError;	

    // Get the device information for this handset.
    // Reference all the data by looking at the pMe->DeviceInfo structure
    // Check the API reference guide for all the handy device info you can get
    pMe->DeviceInfo.wStructSize = sizeof(pMe->DeviceInfo);
    ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->DeviceInfo);
	SETAEERECT(&pMe->aRect, 0, 0, pMe->DeviceInfo.cxScreen, pMe->DeviceInfo.cyScreen);
	
	// IHtmlViewerCX^X̍쐬
	pMe->m_pHtmlViewer = NULL;
	nError = ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HTML, (void**)&pMe->m_pHtmlViewer);
	if( nError != SUCCESS )
	{
		return FALSE;
	}

	// IFileMgrCX^X̍쐬
	pMe->m_pFileMgr = NULL;
	nError = ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void**)&pMe->m_pFileMgr);
	if( nError != SUCCESS )
	{
		return FALSE;
	}



    // if there have been no failures up to this point then return success
    return TRUE;
}

// this function is called when your application is exiting
void TestHtmlViewer_FreeAppData(TestHtmlViewer* pMe)
{
    // insert your code here for freeing any resources you have allocated...

    // example to use for releasing each interface:
    // if ( pMe->pIMenuCtl != NULL )         // check for NULL first
    // {
    //    IMENUCTL_Release(pMe->pIMenuCtl)   // release the interface
    //    pMe->pIMenuCtl = NULL;             // set to NULL so no problems trying to free later
    // }
    //
	if( pMe->m_pHtmlViewer != (IHtmlViewer*)0 )
	{
		IHTMLVIEWER_Release(pMe->m_pHtmlViewer);
		pMe->m_pHtmlViewer = 0;
	}

	if( pMe->m_pFileMgr != (IFileMgr*)0 )
	{
		IFILEMGR_Release(pMe->m_pFileMgr);
		pMe->m_pFileMgr = 0;
	}

}

// LoadFiles :: t@C̓ǂݍݏ
void LoadFiles(TestHtmlViewer *pApp, char *pFileName)
{
	// ƕϐ̒`
	IFile			*pFile = NULL;

	// NULL`FbN
	if( pApp == NULL )
	{
		DBGPRINTF("pApp is NULL");
		return;
	}

	if( pApp->m_pFileMgr == NULL )
	{
		DBGPRINTF("IFileMgrCX^X is NULL");
		return;
	}

	pFile = IFILEMGR_OpenFile(pApp->m_pFileMgr, pFileName, _OFM_READ);
	if( pFile == NULL )
	{
		return;
	}
	// IHtmlViewerANeBuɐݒ
	IHTMLVIEWER_SetRect(pApp->m_pHtmlViewer, &pApp->aRect);
	IHTMLVIEWER_SetActive(pApp->m_pHtmlViewer, TRUE);
	// ʕ`
	IHTMLVIEWER_LoadStream(pApp->m_pHtmlViewer, (IAStream*)pFile);
	// IHtmlViewr̃R[obN֐̓o^
	IHTMLVIEWER_SetNotifyFn(pApp->m_pHtmlViewer, (PFNHVIEWNOTIFY)HtmlNotify_MENU, pApp);

	// [
	IFILE_Release(pFile);
	pFile = NULL;

	return;
}

void HtmlNotify_MENU(TestHtmlViewer *pApp, HViewNotify *phvn, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
	// NULL`FbN
	if( pApp == NULL )
	{
		DBGPRINTF("pApp is NULL");
		return;
	}

	if( pApp->m_pHtmlViewer == NULL )
	{
		DBGPRINTF("IHtmlViewerCX^X is NULL");
		return;
	}

	// ʐpo̓obt@̏
	//*(pApp->m_szOutput) = '\0';

	switch( phvn->code )
	{
	case HVN_JUMP:
		switch( ATOI( phvn->u.submit.pszURL ) )
		{
		default :
			break;
		}
		break;

	case HVN_SUBMIT:
		switch( ATOI( phvn->u.submit.pszURL ) )
		{
		case 101:
			//my_MakeVoiceCall(pApp, phvn->u.submit.pszData);
			break;
		
		default:
			break;
		}
		break;

	default:
		break;
	}

	return;
}