#ifndef _FMTUNER_H_
#define _FMTUNER_H_
/*======================================================
FILE:  FMTUNER.h

SERVICES:
   SANYO FMTUNER Interface

DESCRIPTION:
   This file provides the interface specification for
   the SANYO PHONESYSTEM Interface

CLASS IDs:

NOTES:


=====================================================*/
/*-----------------------------------------------------------------------------
                        EDIT HISTORY FOR MODULE

No.      when      who          what, where, why
001		04/10/20	dan@cresco	background ctrl

--------  --------  -----------  -----------------------------------------------
--------------------------------------------------------------------------------*/
#include "AEE.h"


typedef struct _IFMTUNER IFMTUNER;

//
// Command codes
//
//
#define FMT_CMD_BASE           1        // Command base used by IMedia
#define FMT_CMD_USER_BASE      0x100    // Command base for derived class

#define FMT_CMD_SEEK           (FMT_CMD_BASE + 1)    // Seek()
#define FMT_CMD_SEEKSTOP       (FMT_CMD_BASE + 2)    // SeekStop()
#define FMT_CMD_GETVOLUME      (FMT_CMD_BASE + 3)    // GetVolume()
#define FMT_CMD_SETVOLUME      (FMT_CMD_BASE + 4)    // SetVolume()
#define FMT_CMD_SETTUNE        (FMT_CMD_BASE + 5)    // SetTune()
#define FMT_CMD_GETTUNE        (FMT_CMD_BASE + 6)    // GetTune()
#define FMT_CMD_GETSTATUS      (FMT_CMD_BASE + 7)    // GetStatus()
#define FMT_CMD_CTRLPOWER      (FMT_CMD_BASE + 8)    // CtrlPower()
#define FMT_CMD_HEADSET        (FMT_CMD_BASE + 9)    // headset
#define FMT_CMD_CLOSEAPP       (FMT_CMD_BASE + 10)   // CLOSE application
#define FMT_CMD_DISABLEALARM   (FMT_CMD_BASE + 11)   // alarm disable
#define FMT_CMD_MANNER         (FMT_CMD_BASE + 12)   // MANNER mode 
// PTX803_FMradio_background_Add -s-
#define FMT_CMD_SEEK_BG        (FMT_CMD_BASE + 13)   // BGAvpSeek()
#define FMT_CMD_GETVOLUME_BG   (FMT_CMD_BASE + 14)   // BGAvpGetVolume()
#define FMT_CMD_SETVOLUME_BG   (FMT_CMD_BASE + 15)   // BGAvpSetVolume()
#define FMT_CMD_SETTUNE_BG     (FMT_CMD_BASE + 16)   // BGAvpSetTune()
#define FMT_CMD_GETTUNE_BG     (FMT_CMD_BASE + 17)   // BGAvpGetTune()
// PTX803_FMradio_background_Add -s-

//
// Status codes
//
#define FMT_STATUS_BASE              1     // Status used by IMedia
#define FMT_STATUS_USER_BASE         0x100 // Status base for derived class

#define FMT_STATUS_START             (FMT_STATUS_BASE)        // [FMTuner] started
#define FMT_STATUS_DONE              (FMT_STATUS_BASE + 1)    // [FMTuner] Command completed
#define FMT_STATUS_ABORT             (FMT_STATUS_BASE + 2)    // [FMTuner] Command aborted
#define FMT_STATUS_SEEK              (FMT_STATUS_BASE + 3)    // [FMTuner] seek done
#define FMT_STATUS_SEEK_FAIL         (FMT_STATUS_BASE + 4)    // [FMTuner] seek cmd failed
#define FMT_STATUS_SEEK_STOP         (FMT_STATUS_BASE + 5)    // [FMTuner] NOT USED
#define FMT_STATUS_SEEK_NOW          (FMT_STATUS_BASE + 6)    // [FMTuner] seek cmd now freq
#define FMT_STATUS_SEEKSTOP          (FMT_STATUS_BASE + 7)    // [FMTuner] seekstop done
#define FMT_STATUS_SEEKSTOP_FAIL     (FMT_STATUS_BASE + 8)    // [FMTuner] seekstop failed
#define FMT_STATUS_GETVOLUME         (FMT_STATUS_BASE + 9)    // [FMTuner] getvolume done
#define FMT_STATUS_GETVOLUME_FAIL    (FMT_STATUS_BASE + 10)   // [FMTuner] getvolume failed
#define FMT_STATUS_SETVOLUME         (FMT_STATUS_BASE + 11)   // [FMTuner] setvolume done
#define FMT_STATUS_SETVOLUME_FAIL    (FMT_STATUS_BASE + 12)   // [FMTuner] setvolume failed
#define FMT_STATUS_GETTUNE           (FMT_STATUS_BASE + 13)   // [FMTuner] gettune done
#define FMT_STATUS_GETTUNE_FAIL      (FMT_STATUS_BASE + 14)   // [FMTuner] gettune failed
#define FMT_STATUS_SETTUNE           (FMT_STATUS_BASE + 15)   // [FMTuner] settune done
#define FMT_STATUS_SETTUNE_FAIL      (FMT_STATUS_BASE + 16)   // [FMTuner] settune failed
#define FMT_STATUS_GETSTATUS         (FMT_STATUS_BASE + 17)   // [FMTuner] getstatus done
#define FMT_STATUS_GETSTATUS_FAIL    (FMT_STATUS_BASE + 18)   // [FMTuner] getstatus failed
#define FMT_STATUS_CTRLPOWER         (FMT_STATUS_BASE + 19)   // [FMTuner] ctrlpower done
#define FMT_STATUS_CTRLPOWER_FAIL    (FMT_STATUS_BASE + 20)   // [FMTuner] ctrlpower failed
#define FMT_STATUS_HEADSET           (FMT_STATUS_BASE + 21)   // [FMTuner] headset status get done
#define FMT_STATUS_CLOSEAPP          (FMT_STATUS_BASE + 22)   // [FMTuner] close fmtuner applet done
#define FMT_STATUS_DISABLEALARM      (FMT_STATUS_BASE + 23)   // [FMTuner] disable alarm
#define FMT_STATUS_MANNER            (FMT_STATUS_BASE + 24)   // [FMTuner] manner mode

#define FMT_PENDING                  0xEFFFFFFF               // [FMTuner] pending


// [INFOR}h
#define FMT_INFO_BASE           1        
#define FMT_INFO_USER_BASE      0x100    

#define FMT_INFO_HEADSET       (FMT_INFO_BASE + 1)    // HEADSET INFO
#define FMT_INFO_FLIP          (FMT_INFO_BASE + 2)    // FLIP INFO
#define FMT_INFO_ROAMING       (FMT_INFO_BASE + 3)    // ROAMING INFO

// 萔`
#define FMT_DEF_HEADSET_ON     1
#define FMT_DEF_HEADSET_OFF    0
#define FMT_DEF_FLIP_OPEN      0
#define FMT_DEF_FLIP_CLOSE     1
#define FMT_DEF_POWER_ON       1
#define FMT_DEF_POWER_OFF      0
#define FMT_DEF_STEREO         0
#define FMT_DEF_MONO           1
#define FMT_DEF_MANNER_OFF     0
#define FMT_DEF_MANNER_ON      1
#define FMT_DEF_ROAMING_OFF    0
#define FMT_DEF_ROAMING_ON     1



typedef struct AEEFMTunerCmdNotify {
   AEECLSID    clsFMTuner;   // CLSID of IFMTuner concrete class
   IFMTUNER   *pIFMTuner;    // Pointer to IFMTuner
   int         nCmd;         // Command code
   int         nSubCmd;      // Sub command code, if any or 0
   int         nStatus;      // Status code
   void *      pCmdData;     // Contains one of the values listed above
   uint32      dwSize;       // Size of pCmdData
} AEEFMTunerCmdNotify;

typedef void (*PFNFMTUNERNOTIFY)(void * pUser, AEEFMTunerCmdNotify *pCmdNotify);


#define INHERIT_IFMTuner(iname) \
   INHERIT_IQueryInterface(iname);\
    \
    int     (*RegisterNotify)(iname *p, PFNFMTUNERNOTIFY pfnNotify, void *pUser);\
    \
    int     (*Seek)(iname *p, int dir, uint32 *pFreq);\
    int     (*SeekStop)(iname *p);\
    int     (*SetVolume)(iname *p, int vol);\
    int     (*GetVolume)(iname *p, int *pVol);\
    int     (*SetTune)(iname *p, uint32 Freq, int *pSen);\
    int     (*GetTune)(iname *p, uint32 *pFreq);\
    int     (*GetState)(iname *p, int *pSen);\
    int     (*CtrlPower)(iname *p, int Power);\
    int     (*ChangeDevice)(iname *p, int dev);\
    int     (*GetInfo)(iname *p, int para, int *pInfo );\
    int     (*SetMode)(iname *p, int mode);\
    \
    int     (*Record)(iname *p);\
    int     (*RecStop)(iname *p);\
    int     (*GetFileName)(iname *p, unsigned char *name, int *wSize)
QINTERFACE(IFMTUNER)
{
   INHERIT_IFMTuner(IFMTUNER);
};




#define     IFMTUNER_AddRef(p)                        GET_PVTBL(p,IFMTUNER)->AddRef(p)
#define     IFMTUNER_Release(p)                       GET_PVTBL(p,IFMTUNER)->Release(p)
#define     IFMTUNER_QueryInterface(p,c,i)            GET_PVTBL(p,IFMTUNER)->QueryInterface(p,c,i)
#define     IFMTUNER_Seek(p,dir,pFreq)                GET_PVTBL(p,IFMTUNER)->Seek(p,dir,pFreq)
#define     IFMTUNER_SeekStop(p)                      GET_PVTBL(p,IFMTUNER)->SeekStop(p)
#define     IFMTUNER_SetVolume(p,vol)                 GET_PVTBL(p,IFMTUNER)->SetVolume(p,vol)
#define     IFMTUNER_GetVolume(p,pVol)                GET_PVTBL(p,IFMTUNER)->GetVolume(p,pVol)
#define     IFMTUNER_SetTune(p,Freq,pSen)             GET_PVTBL(p,IFMTUNER)->SetTune(p,Freq,pSen)
#define     IFMTUNER_GetTune(p,pFreq)                 GET_PVTBL(p,IFMTUNER)->GetTune(p,pFreq)
#define     IFMTUNER_GetState(p,pSen)                 GET_PVTBL(p,IFMTUNER)->GetState(p,pSen)
#define     IFMTUNER_CtrlPower(p,power)               GET_PVTBL(p,IFMTUNER)->CtrlPower(p,power)
#define     IFMTUNER_ChangeDevice(p,dev)              GET_PVTBL(p,IFMTUNER)->ChangeDevice(p,dev)
#define     IFMTUNER_RegisterNotify(p,pFn,pUser)      GET_PVTBL(p,IFMTUNER)->RegisterNotify(p,pFn,pUser)
#define     IFMTUNER_GetInfo(p,para,info)             GET_PVTBL(p,IFMTUNER)->GetInfo(p,para,info)
#define     IFMTUNER_SetMode(p,mode)                  GET_PVTBL(p,IFMTUNER)->SetMode(p,mode)
//PTX805 Cui Xinyu add For UIQ401 20050916-S-
#define     IFMTUNER_Record(p)                  GET_PVTBL(p,IFMTUNER)->Record(p)
#define     IFMTUNER_RecStop(p)                  GET_PVTBL(p,IFMTUNER)->RecStop(p)
#define     IFMTUNER_GetFileName(p, name, wSize)                  GET_PVTBL(p,IFMTUNER)->GetFileName(p, name, wSize)
//PTX805 Cui Xinyu add For UIQ401 20050916-E-
/*
=======================================================================
  INTERFACES   DOCUMENTATION
=======================================================================
*/

/*=====================================================================

֐: IFMTUNER_AddRef()

@\:
   ̊֐ IFMTUNERC^tF[XIuWFNg̎QƃJEg
   ₷B͌ĂяȍꍇɃIuWFNgL邱Ƃe
   BIuWFNg͎QƃJEgOi[jɒBƂɉB
   IFMTUNER_Release()QƂ̂ƁB

vg^Cv:
   uint32  IFMTUNER_AddRef
   (
      IFMTUNER  *pIFMTUNER
   )

p[^:
   pIFMTUNER    IFMTUNERC^tF[XIuWFNgւ̃|C^

߂l:
   ̃IuWFNgɑ΂QƃJEg

:
   IuWFNgLłΐ̒lԂ

p:
   Ȃ

֘A:
   IFMTUNER_Release()

=======================================================================

֐: IFMTUNER_Seek()

@\:
   M\gV[NB

vg^Cv:
   int IFMTUNER_Seek
   (
      IFMTUNER  * pIFMTUNER,
      int         direction,
      uint32    * pFrequency
   )

p[^:
   pIFMTUNER [in]      IFMTUNERC^tF[XIuWFNgւ̃|C^
   direction [in]      SEEKi{̓CN^A|̓fN^j
   pFrequency[in/out]  MJngi͂ŃZbgꂽgV[NJnj
                       CALLBACKݒ肳ĂȂꍇ́AMg߂B

߂l:
   SUCCESS             Iiɖ߂󂯕tj
   EBADPARAM           sȏꍇ
   EBADSTATE           IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   ̂`ohgpɂ͑OɁAIFMTUNER_RegisterNotify()֐ɂCALLBACK[`
   o^邱ƁBiV[NɎԂvׁA񓯊ɂׁj

p:
   Ȃ

֘A:
   IFMTUNER_RegisterNotify()

=======================================================================

֐: IFMTUNER_SeekStop()

@\:
   FM`[i[SEEK𒆎~

vg^Cv:

   int IFMTUNER_SeekStop
   (
      IFMTUNER      *pIFMTUNER
   )

p[^:
   pIFMTUNER [in]   IFMTUNERC^tF[XIuWFNgւ̃|C^

߂l:
   SUCCESS              I
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   Ȃ

p:
   Ȃ

֘A:
   Ȃ

=======================================================================

֐: IFMTUNER_SetVolume()

@\:
   WIo͂鉹ʂZbg

vg^Cv:
   int IFMTUNER_SetVolume
   (
      IFMTUNER      *pIFMTUNER,
      int           Volume
   )

p[^:
   pIFMTUNER [in]   IFMTUNERC^tF[XIuWFNgւ̃|C^
   Volume    [in]   ʁiO`POOj

߂l:
   SUCCESS              
   EBADPARM             ̓p[^s
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   ʒĺAO`POOLƂAOMUTEƂ

p:
   Ȃ

֘A:
   Ȃ

=======================================================================

֐: IFMTUNER_GetVolume()

@\:
   ݐݒ肳Ă郉WȈo͉ʂ擾B

vg^Cv:
   int IFMTUNER_GetVolume
   (
      IFMTUNER      *pIFMTUNER,
      int           *pVolume
   )

p[^:
   pIFMTUNER [in]       IFMTUNERC^tF[XIuWFNgւ̃|C^
   pVolume   [out]      擾ʒl

߂l:
   SUCCESS              
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   ̂`ohgpɂ͑OɁAIFMTUNER_RegisterNotify()֐ɂCALLBACK[`
   o^邱ƁBiʎ擾ɎԂvׁA񓯊ɂׁj

p:
   Ȃ

֘A:
   IFMTUNER_RegisterNotify()

=======================================================================

֐: IFMTUNER_SetTune()

@\:
   wgɃ`[jOsB

vg^Cv:
   int IFMTUNER_SetTune
   (
      IFMTUNER  *pIFMTUNER,
      uint32     Frequency,
      int       *pSensitivity
   )

p[^:
   pIFMTUNER [in]       IFMTUNERC^tF[XIuWFNgւ̃|C^
   Frequency [in]       ݒg
   pSensitivity [out]   Mx

߂l:
   SUCCESS              
   EBADPARM             ̓p[^s
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   ̂`ohgpɂ͑OɁAIFMTUNER_RegisterNotify()֐ɂCALLBACK[`
   o^邱ƁBi`[jOɎԂvׁA񓯊ɂׁj

p:
   Ȃ

֘A:
   IFMTUNER_RegisterNotify()

=======================================================================

֐: IFMTUNER_GetTune()

@\:
   ݐݒ肳Ă`[i[̎g擾B

vg^Cv:
   int IFMTUNER_GetTune
   (
      IFMTUNER  *pIFMTUNER,
      uint32    *pFrequency
   )

p[^:
   pIFMTUNER  [in]    IFMTUNERC^tF[XIuWFNgւ̃|C^
   pFrequency [out]   ݒg

߂l:
   SUCCESS              
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   ̂`ohgpɂ͑OɁAIFMTUNER_RegisterNotify()֐ɂCALLBACK[`
   o^邱ƁBiݒg擾ɎԂvׁA񓯊ɂׁj

p:
   Ȃ

֘A:
   IFMTUNER_RegisterNotify()

=======================================================================

֐: IFMTUNER_GetState()

@\:
   el`[i[̎Mx擾܂

vg^Cv:
   int IFMTUNER_GetState
   (
      IFMTUNER  *pIFMTUNER,
      int       *pSensitivity
   )

p[^:
   pIFMTUNER    [in]  IFMTUNERC^tF[XIuWFNgւ̃|C^
   pSensitivity [out] Mx

߂l:
   SUCCESS              
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   ̂`ohgpɂ͑OɁAIFMTUNER_RegisterNotify()֐ɂCALLBACK[`
   o^邱ƁBiݒg擾ɎԂvׁA񓯊ɂׁj

p:
   Ȃ

֘A:
   IFMTUNER_RegisterNotify()

=======================================================================

֐: IFMTUNER_CtrlPower()

@\:
   el`[i[̓dsB

vg^Cv:
   int IFMTUNER_CtrlPower
   (
      IFMTUNER  *pIFMTUNER,
      int       Power
   )

p[^:
   pIFMTUNER [in]   IFMTUNERC^tF[XIuWFNgւ̃|C^
   Power     [in]   diOFnee^PFnmj

߂l:
   SUCCESS              
   EBADPARM             ̓p[^s
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   Ȃ

p:
   Ȃ

֘A:
   Ȃ

=======================================================================

֐: IFMTUNER_ChangeDevice()

@\:
   ړ@̏̒ʘb͒M̌擾܂

vg^Cv:
   int IFMTUNER_ChangeDevice
   (
      IFMTUNER  *pIFMTUNER,
      int        Device
   )

p[^:
   pIFMTUNER [in]   IFMTUNERC^tF[XIuWFNgւ̃|C^
   Device    [out]  o̓foCX

߂l:
   SUCCESS              
   EBADPARM             ̓p[^s
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
  T|[gfoCX́ACtĤ݂ƂB

p:
   Ȃ

֘A:
   AEESoundDevice

=======================================================================

֐: IFMTUNER_RegisterNotify()

@\:
   el`[i[֘ÃR[obN֐o^܂B

vg^Cv:
   int IFMTUNER_RegisterNotify
   (
      IFMTUNER         *pIFMTUNER,
      PFNFMTUNERNOTIFY  pfnNotify,
      void             *pUser
   )

p[^:
   pIFMTUNER [in]   IFMTUNERC^tF[XIuWFNgւ̃|C^
   pfnNotify [in]   R[obN֐ւ̃|C^
   pUser     [in]   R[obN鎞Ɏgp郆[U|f[^

߂l:
   SUCCESS              
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   񓯊ʒmCxgΉR[obN[`o^B

p:
   Ȃ

֘A:
   Ȃ

=======================================================================

֐: IFMTUNER_Release()

@\:
   ̊֐ IFMTUNER C^tF[XIuWFNg̎QƃJEg
   炷BQƃJEgOi[jɒBAIuWFNg̓
   AƂȂB

vg^Cv:
   uint32 IFMTUNER_Release
   (
      IFMTUNER  *pIFMTUNER,
   )

p[^:
   pIFMTUNER    IFMTUNERC^tF[XIuWFNgւ̃|C^

߂l:
   QƃJEg         ̃IuWFNgɑ΂QƃJEg
   Oi[j           IuWFNgɊJꖳƂȂĂꍇ

:
   Ȃ

p:
   Ȃ

֘A:
   IFMTUNER_AddRef()

=======================================================================

֐: IFMTUNER_GetInfo()

@\:
   ̊֐ BREWŃT|[gĂȂ[̏Ԃ擾B

vg^Cv:
   uint IFMTUNER_GetInfo
   (
      IFMTUNER  *pIFMTUNER,
      int        para,
      int       *pInfo
   )

p[^:
   pIFMTUNER    IFMTUNERC^tF[XIuWFNgւ̃|C^
   int          vp[^
   info         ʒm

߂l:

   SUCCESS              
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ

:
   Ȃ

p:
   Ȃ

֘A:
   Ȃ

=======================================================================

֐: IFMTUNER_SetMode()

@\:
   ̊֐ FMWĨXeI^mMODE؂ւsB

vg^Cv:
   uint IFMTUNER_SetMode
   (
      IFMTUNER  *pIFMTUNER,
      int        mode
   )

p[^:
   pIFMTUNER    IFMTUNERC^tF[XIuWFNgւ̃|C^
   int          [hiXeI^mj

߂l:

   SUCCESS              
   EBADSTATE            IFMTUNERC^tF[XIuWFNgfBԂłȂ
   EBADPARAM            sȏꍇ

:
   Ȃ

p:
   Ȃ

֘A:
   Ȃ

=======================================================================
*/

#endif //_FMTUNER_H_
